// Desktop Rox - A game by Stephen Stair (sgstair@akkit.org) for LudumDare 11 (48h game competition)
// WindowSpriteManager - tool for rendering sprites within a window, for testing purposes (and as a fallback if the desktop version doesn't work)

#include "WindowSpriteManager.h"
#include "BitmapSprite.h"

extern HINSTANCE g_hInst;

HWND WindowSpriteManager::hWnd;
Bitmap WindowSpriteManager::cachebmp;

vector<WindowSpriteRecord> WindowSpriteManager::mysprites;

WindowSpriteManager::WindowSpriteManager(int width, int height)
{
	WNDCLASS wc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hbrBackground=0;
	wc.hCursor=LoadCursor(NULL,MAKEINTRESOURCE(IDC_ARROW));
	wc.hIcon=0;
	wc.hInstance=g_hInst;
	wc.lpfnWndProc=WindowSpriteManager::WndProc;
	wc.lpszClassName="WSM_class";
	wc.lpszMenuName=0;
	wc.style=CS_HREDRAW|CS_VREDRAW;
	RegisterClass(&wc);

	cachebmp.resize(width,height);
		
	RECT rc;
	rc.top = rc.left = 0;
	rc.bottom = height;
	rc.right = width;
	AdjustWindowRect(&rc,WS_OVERLAPPEDWINDOW,false);

	hWnd = CreateWindow("WSM_class","WindowSpriteManager",WS_OVERLAPPEDWINDOW, 10, 10, rc.right-rc.left, rc.bottom-rc.top, 0, 0, g_hInst, 0 );

	ShowWindow(hWnd, SW_SHOW);

}
WindowSpriteManager::~WindowSpriteManager(void)
{
	DestroyWindow(hWnd);
	mysprites.clear();
}


int WindowSpriteManager::GetFieldWidth()
{
	return cachebmp.wid();
}
int WindowSpriteManager::GetFieldHeight()
{
	return cachebmp.hei();
}
vector<BoundaryPoint> WindowSpriteManager::GetFieldBoundary()
{  // boundary points traverse the screen(s) CLOCKWISE
	vector<BoundaryPoint> bps;
	bps.resize(4);
	bps[0].x = 0;
	bps[0].y = 0;
	bps[1].x = cachebmp.wid();
	bps[1].y = 0;
	bps[2].x = cachebmp.wid();
	bps[2].y = cachebmp.hei();
	bps[3].x = 0;
	bps[3].y = cachebmp.hei();
	return bps;
}
RECT WindowSpriteManager::GetMainWindow()
{
	RECT rc;
	rc.top = 0;
	rc.left = 0;
	rc.bottom = cachebmp.hei();
	rc.right = cachebmp.wid();
	return rc;
}

void WindowSpriteManager::SetClickHandler(fptrClickHandler h)
{
	myhandler = h;
}

void WindowSpriteManager::AddSprite( BitmapSprite * sprite )
{
	RemoveSprite(sprite); // precautionary
	mysprites.push_back(WindowSpriteRecord());
	mysprites.back().sprite = sprite;
}
bool WindowSpriteManager::RemoveSprite( BitmapSprite * sprite )
{
	size_t i;
	for(i=0;i<mysprites.size();i++)
	{
		if(mysprites[i].sprite==sprite)
		{
			mysprites.erase( mysprites.begin() + i );
			return true;
		}
	}
	return false;
}

void WindowSpriteManager::Redraw()
{
	// rerender sprites
	cachebmp.cls(Color(64,64,64));
	size_t i;
	for(i=0;i<mysprites.size();i++)
	{
		BitmapSprite * bs;
		bs = mysprites[i].sprite;
		if(bs->alpha!=0)
			cachebmp.render( bs->bx, bs->by, bs->bmp );
	}

	// invalidate window to repaint
	RECT rc;
	GetClientRect(hWnd,&rc);
	InvalidateRect(hWnd,&rc,false);
}

fptrClickHandler WindowSpriteManager::myhandler = 0;

LRESULT WINAPI WindowSpriteManager::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_CREATE:
		return 0;
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;

	case WM_LBUTTONDOWN:
		{
			int x = LOWORD(lParam);
			int y = HIWORD(lParam);
			size_t i;
			if(myhandler)
			{
				for(i=0;i<mysprites.size();i++)
				{
					if(mysprites[i].sprite->flags&FLAGS_WANTCLICKS)
					{
						if(x>=mysprites[i].sprite->bx && y>=mysprites[i].sprite->by)
						myhandler(x-mysprites[i].sprite->bx,y-mysprites[i].sprite->by,mysprites[i].sprite);
					}
				}
			}
		}
		return 0;

	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hDC = BeginPaint(hWnd, &ps);
			BitBlt(hDC,0,0,cachebmp.wid(),cachebmp.hei(),cachebmp.DC(),0,0,SRCCOPY);
			EndPaint(hWnd,&ps);
		}
		return 0;
	}
	return DefWindowProc(hWnd,msg,wParam,lParam);
}

//static HWND hWnd;
//static Bitmap cachebmp;
//vector<WindowSpriteRecord> mysprites;
